package cn.ecpark.service.usergw.biz.filters.factory;

import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

import com.alibaba.fastjson.JSON;

import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.http.HttpMethod;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.util.MultiValueMap;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;;

@Component
@Slf4j
public class DubboGatewayFilterFactory extends AbstractGatewayFilterFactory<DubboGatewayFilterFactory.Config> {

    @Autowired
    private ApplicationContext appContext;

    /**
     * DUBBO_URI key.
     */
    public static final String DUBBO_URI = "dubbo_uri";
    public static HashMap<String, List<String>> mehtods = new HashMap<>();

    public DubboGatewayFilterFactory() {
        super(Config.class);
    }

    @Override
    public List<String> shortcutFieldOrder() {
        return Arrays.asList(DUBBO_URI);
    }

    @Override
    public GatewayFilter apply(Config config) {
        String uri = config.dubboUri;

        return (exchange, chain) -> {

            GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
            GenericService gs = gsPool.get(uri);
            
            ServerHttpRequest req = exchange.getRequest();
            Flux<DataBuffer> body = req.getBody();
 
            MultiValueMap<String, String> queryParams = req.getQueryParams();
     

            // body.map( buffer ->  {
            //     byte[] bytes = new byte[buffer.readableByteCount()];
            //     buffer.read(bytes);
            //     DataBufferUtils.release(buffer);
            //     try {
            //         String bodyString = new String(bytes, "utf-8");
            //         log.error("test body");
            //     } catch (Exception e) {
            //         e.printStackTrace();
            //     }
            //     return Flux.just(bytes);
            // });

            List<String> methodString = queryParams.get("method");
            List<String> params = queryParams.get("params");
            if(methodString.size() != 0) {
                // special
                List<String> paramTypes = mehtods.get(methodString.get(0));
                Object result = gs.$invoke(methodString.get(0),  Arrays.copyOf(paramTypes.toArray(), paramTypes.size(), String[].class), params.toArray());
                if (result != null) {
                    ServerHttpResponse response = exchange.getResponse();
                    return response.writeWith(Mono
                            .just(response.bufferFactory().wrap(ByteBuffer.wrap(JSON.toJSONString(result).getBytes()))));
                }
            } else {
                log.info("queryParams.get(\"method\") is null");
            }


            return chain.filter(exchange);
        };
    }

    public static class Config {

        private String dubboUri;

        public String getDubboUri() {
            return dubboUri;
        }

        public void setDubboUri(String dubboUri) {
            this.dubboUri = dubboUri;
        }

    }

}