package cn.ecpark.service.usergw.config;

import java.io.InputStream;
import java.lang.reflect.Method;
import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.function.BiConsumer;

import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.FilterDefinition;
import org.springframework.cloud.gateway.handler.predicate.PredicateDefinition;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.cloud.gateway.route.RouteDefinitionLocator;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.stereotype.Component;
import org.yaml.snakeyaml.Yaml;

import cn.ecpark.service.usergw.biz.filters.bean.GenericServicePool;
import cn.ecpark.service.usergw.biz.filters.factory.DubboGatewayFilterFactory;
import cn.ecpark.service.usergw.utils.Convert;
import cn.ecpark.service.usergw.utils.Extract;
import lombok.extern.slf4j.Slf4j;
import reactor.core.publisher.Flux;

/**
 * ConfigGateway
 */
@Component
@Slf4j
public class ConfigGateway implements RouteDefinitionLocator {

  // List<MediaType> mediaTypes = new ArrayList<>();

  List<FilterDefinition> defaultFilters = new ArrayList<>();
  HashMap<String, BiConsumer<ReferenceConfig<GenericService>, Object>> specialField = new HashMap<String, BiConsumer<ReferenceConfig<GenericService>, Object>>();
  Set<String> ignoreKey = new HashSet<>();

  @Autowired
  private ApplicationContext appContext;

  public ConfigGateway() {
    specialField.put("application", ConfigSpecialFunction::setApplication);
    specialField.put("registry", ConfigSpecialFunction::setRegistry);
    specialField.put("methods", ConfigSpecialFunction::setMethods);

    ignoreKey.add("predicates");
    ignoreKey.add("filters");
  }

  @Override
  @SuppressWarnings("unchecked")
  public Flux<RouteDefinition> getRouteDefinitions() {
    // WebFluxConfigurationSupport a;
    defaultFilters.clear();
    Object inputStream = null;
    String[] gatewayConfigPathList = { "gateway.yml", "gateway.yaml", "Gateway.yml", "Gateway.yaml" };
    for (String gatewayConfigPath : gatewayConfigPathList) {
      inputStream = this.getClass().getClassLoader().getResourceAsStream(gatewayConfigPath);
      if (inputStream != null) {
        break;
      }
    }

    if (inputStream != null) {
      Map<String, Object> configYaml = new Yaml().load((InputStream) inputStream);
      if (configYaml != null) {

        // streaming-media-types 这个先不管
        // NettyWriteResponseFilter nwrf =
        // applicationContext.getBean(NettyWriteResponseFilter.class);
        // try {
        // Field field = nwrf.getClass().getDeclaredField("streamingMediaTypes");
        // field.setAccessible(true);
        // List<MediaType> mts = new ArrayList<MediaType>();
        // mts.add(MediaType.TEXT_EVENT_STREAM);
        // mts.add(MediaType.APPLICATION_STREAM_JSON);
        // field.set(nwrf, mts);
        // } catch (Exception e) {
        // log.error(e.toString());
        // }

        Map<String, Object> defaultYaml = (Map<String, Object>) configYaml.get("default");
        Map<String, Object> dubboYaml = (Map<String, Object>) configYaml.get("dubbo");

        List<RouteDefinition> routeList = new ArrayList<RouteDefinition>();

        if (defaultYaml != null && !defaultYaml.isEmpty()) {
          this.getDefaultFilter(defaultFilters, defaultYaml);
          this.configDefault(routeList, defaultYaml);
        }

        if (dubboYaml != null && !dubboYaml.isEmpty()) {
          this.configHttp2Dubbo(routeList, dubboYaml);
        }

        if (!routeList.isEmpty()) {
          return Flux.fromIterable(routeList);
        }

      }
    }

    return Flux.empty();
    // return Flux.fromIterable(it);
  }

  

  @SuppressWarnings("unchecked")
  private void configDefault(List<RouteDefinition> routeList, Map<String, Object> defaultYaml) {
    if (defaultYaml != null) {

      Object unknownRoutes = defaultYaml.get("routes");
      if (unknownRoutes != null) {
        List<LinkedHashMap<String, List<String>>> routes = (ArrayList<LinkedHashMap<String, List<String>>>) unknownRoutes;
        for (LinkedHashMap<String, List<String>> iter : routes) {

          List<FilterDefinition> filters = new ArrayList<>();
          List<PredicateDefinition> predicates = new ArrayList<>();
          RouteDefinition rd = new RouteDefinition();

          // 设置基础属性
          this.parseAndSetBase(rd, iter);

          // predicates: 下的相关属性
          this.parseAndAddPredicates(predicates, iter, "predicates");

          // filters: 下的相关属性
          this.parseAndAddFilters(filters, iter, "filters");

          rd.setPredicates(predicates);
          rd.setFilters(filters);

          routeList.add(rd);
        }
      }
    }
  }

  @SuppressWarnings("unchecked")
  private void configHttp2Dubbo(List<RouteDefinition> routeList, Map<String, Object> configDubbo) {

    try {

      Object unknownRoutes = configDubbo.get("routes");
      if (unknownRoutes != null) {

        List<LinkedHashMap<String, List<String>>> routes = (ArrayList<LinkedHashMap<String, List<String>>>) unknownRoutes;
        for (LinkedHashMap<String, List<String>> iter : routes) {

          List<FilterDefinition> filters = new ArrayList<>();
          List<PredicateDefinition> predicates = new ArrayList<>();
          RouteDefinition rd = new RouteDefinition();

          // 设置基础属性
          String dubboUri = this.parseDubboUriAndSetBase(rd, iter);

          // predicates: 下的相关属性
          this.parseAndAddPredicates(predicates, iter, "predicates");

          // filters: 下的相关属性
          this.parseAndAddDubboFilters(dubboUri, filters, iter, "filters");

          rd.setPredicates(predicates);
          rd.setFilters(filters);

          routeList.add(rd);

        }

      }

    } catch (Exception e) {
      log.error(e.getMessage());
      log.warn("App is exit");
      ((ConfigurableApplicationContext) appContext).close();
    }

    // gs.$invoke(method, parameterTypes, args)

  }

  @SuppressWarnings("unchecked")
  private void getDefaultFilter(List<FilterDefinition> filters, Map<String, Object> defaultYaml) {
    // default-filters: 下的相关设置
    Object unknownDefaultFilters = defaultYaml.get("default-filters");

    if (unknownDefaultFilters != null) {
      List<String> defaultFiltersYaml = (ArrayList<String>) unknownDefaultFilters;
      for (String filterString : defaultFiltersYaml) {
        filters.add(new FilterDefinition(filterString));
      }
    }
  }

  private void parseAndSetBase(RouteDefinition rd, LinkedHashMap<String, List<String>> iter) {
    // 设置id
    Object id = iter.get("id");
    if (id != null) {
      rd.setId((String) id);
    }

    // 设置uri
    Object uri = iter.get("uri");
    if (uri != null) {
      rd.setUri(URI.create((String) uri));
    }

    // 设置uri
    Object order = iter.get("order");
    if (order != null) {
      rd.setOrder((int) order);
    }
  }

  private String parseDubboUriAndSetBase(RouteDefinition rd, LinkedHashMap<String, List<String>> iter) {
    // 设置id
    Object id = iter.get("id");
    if (id != null) {
      rd.setId((String) id);
    }
    iter.remove("id");

    // 设置uri
    Object uri = iter.get("uri");
    if (uri != null) {
      try {
        rd.setUri(URI.create((String) uri));
      } catch (Exception e) {
        log.warn("dubbo Uri error");
        rd.setUri(URI.create("dubbo://yame"));
      }
    } else {
      rd.setUri(URI.create("dubbo://yame"));
    }
    iter.remove("uri");

    // 设置uri
    Object order = iter.get("order");
    if (order != null) {
      rd.setOrder((int) order);
    }
    iter.remove("order");

    if (uri != null) {
      String uriString = (String) uri;
      uriString = uriString.trim();
      if (uriString.startsWith("dubbo://")) { // dubbo://127.0.0.1/interface?
        return uriString;
      }
    }

    ReferenceConfig<GenericService> reference = new ReferenceConfig<GenericService>();
    reference.setConnections(3);

    String UriString = "dubbo://";
    Object application = iter.get("application");
    if (application != null) {
      BiConsumer<ReferenceConfig<GenericService>, Object> doFunc = specialField.get("application");
      if (doFunc != null) {
        doFunc.accept(reference, (String) application);
      }
    }
    iter.remove("application");

    for (Entry<String, List<String>> entry : iter.entrySet()) {
      // Object group = iter.get("group");
      String key = entry.getKey();
      if (ignoreKey.contains(key)) {
        continue;
      }
      Object value = entry.getValue();

      if (value != null) {
        BiConsumer<ReferenceConfig<GenericService>, Object> doFunc = specialField.get((String) key);
        if (doFunc != null) {
          doFunc.accept(reference, value);
        } else {
          try {
            ConfigSpecialFunction.setDefault(reference, key, value);
          } catch (Exception e) {
            log.error("配置解析错误 字段: {}:{}\n{}", key, value, e.toString());
            log.warn("App is exit");
            ((ConfigurableApplicationContext) appContext).close();
          }
        }
      }
    }

    UriString += Extract.getReferenceConfigKey(reference);
    GenericServicePool gsPool = appContext.getBean(GenericServicePool.class);
    reference.setGeneric(true);
    gsPool.put(UriString, reference.get());

    return UriString;
  }

  private void parseAndAddPredicates(List<PredicateDefinition> predicates, LinkedHashMap<String, List<String>> iter,
      String yamlField) {
    List<String> predicatesYaml = iter.get(yamlField);
    if (predicatesYaml != null) {
      for (String predicateString : predicatesYaml) {
        predicates.add(new PredicateDefinition(predicateString));
      }
    }
  }

  private void parseAndAddFilters(List<FilterDefinition> filters, LinkedHashMap<String, List<String>> iter,
      String yamlField) {
    List<String> filtersYaml = iter.get(yamlField);
    if (filtersYaml != null) {

      filters.addAll(defaultFilters);

      for (String filterString : filtersYaml) {
        filters.add(new FilterDefinition(filterString));
      }
    }
  }

  private void parseAndAddDubboFilters(String dubboUri, List<FilterDefinition> filters,
      LinkedHashMap<String, List<String>> iter, String yamlField) {
    List<String> filtersYaml = iter.get(yamlField);

    filters.addAll(defaultFilters);
    
    if (filtersYaml != null) {
      for (String filterString : filtersYaml) {
        FilterDefinition fd = new FilterDefinition(filterString);
        log.info(fd.getName());
        if (!fd.getName().equals("Dubbo")) {
          filters.add(fd);
        }
      }
    }
    
    filters.add(new FilterDefinition("Dubbo=" + dubboUri));
  }
}