package cn.ecpark.service.usergw.utils;

import org.apache.dubbo.common.utils.StringUtils;
import org.apache.dubbo.config.ReferenceConfig;
import org.apache.dubbo.rpc.service.GenericService;

public class Extract {
    public static String getReferenceConfigKey(ReferenceConfig<GenericService> referenceConfig) {
        String iName = referenceConfig.getInterface();
        if (StringUtils.isBlank(iName)) {
            Class<?> clazz = referenceConfig.getInterfaceClass();
            iName = clazz.getName();
        }
        if (StringUtils.isBlank(iName)) {
            throw new IllegalArgumentException("No interface info in ReferenceConfig" + referenceConfig);
        }

        StringBuilder ret = new StringBuilder();
        ret.append(referenceConfig.getApplication().getName()).append("/");
        if (!StringUtils.isBlank(referenceConfig.getGroup())) {
            ret.append(referenceConfig.getGroup()).append("/");
        }
        ret.append(iName);
        if (!StringUtils.isBlank(referenceConfig.getVersion())) {
            ret.append(":").append(referenceConfig.getVersion());
        }
        return ret.toString();
    }

    public static class GenericServiceBase {

        private String application;
        private String group;
        private String iinterface;
        private String version;

        /**
         * @param application the application to set
         */
        public void setApplication(String application) {
            this.application = application;
        }

        /**
         * @return the application
         */
        public String getApplication() {
            return application;
        }

        /**
         * @return the group
         */
        public String getGroup() {
            return group;
        }

        /**
         * @param group the group to set
         */
        public void setGroup(String group) {
            this.group = group;
        }

        /**
         * @return the iinterface
         */
        public String getInterface() {
            return iinterface;
        }

        /**
         * @param iinterface the iinterface to set
         */
        public void setInterface(String iinterface) {
            this.iinterface = iinterface;
        }

        /**
         * @return the version
         */
        public String getVersion() {
            return version;
        }

        /**
         * @param version the version to set
         */
        public void setVersion(String version) {
            this.version = version;
        }

        public static GenericServiceBase Parse(String dubboUri) {

            // dubbo://application/group/com.a.b:version?method=say
            dubboUri = dubboUri.substring(8);
            String[] token = dubboUri.split("[/:]");

            GenericServiceBase gsb = new GenericServiceBase();
            if (token.length == 3) {
                gsb.setApplication(token[0]);
                gsb.setInterface(token[1]);
                gsb.setVersion(token[2]);
            } else if (token.length == 4) {
                gsb.setApplication(token[0]);
                gsb.setGroup(token[1]);
                gsb.setInterface(token[2]);
                gsb.setVersion(token[3]);
            }

            return gsb;
        }
    }

}